<?php
/**
* Abstract base class for all PHP code elements
*
* @version $Id: Element.php,v 1.1 2003/06/29 12:24:54 hholzgra Exp $
*/

abstract class PECL_Gen_Element extends PEAR {

	/**
	 * Checks wheter a string is a valid PHP type name and returns the official name
	 *
	 * @access public
	 * @param  string Type name
	 * @return string The official type name or boolean false if not a tyle
	 */
	function is_type($name) 
  {
		$types = array("void"     => "void", 
									 "bool"     => "bool", 
									 "boolean"  => "bool", 
									 "int"      => "int", 
									 "integer"  => "int", 
									 "float"    => "float", 
									 "double"   => "float", 
									 "real"     => "float", 
									 "string"   => "string", 
									 "array"    => "array",
									 "object"   => "object",
									 "resource" => "resource",
									 "mixed"    => "mixed",
									 "callback" => "callback",
									 "stream"   => "stream"
									 );
		
		if (isset($types[$name])) {
			return $types[$name];
		} else {
			return false;
		}
	}

	/**
	 * Checks whether a string is a valid PHP name
	 *
	 * @access public
	 * @param  string The name to check
	 * @return bool   true for valid names, false otherwise
	 */
	function is_name($name) 
  {
		if (ereg("^[[:alpha:]_][[:alnum:]_]*$",$name)) {
			// TODO reserved words
			return true;
		} 
		return false;
	}

	/**
	 * Generate C code for element
	 *
	 * @access  public
	 * @return  string C code 
	 */
	abstract function c_code();

	/**
	 * Generate C code header block for all elements of this class
	 *
	 * @access public
	 * @param  string Extension name
	 * @return string C code
	 */
	static function c_code_header($name) 
	{
		return "";
	}

	/**
	 * Generate C code footer block for all elements of this class
	 *
	 * @access public
	 * @param  string Extension name
	 * @return string C code
	 */
	static function c_code_footer($name) 
	{
		return "";
	}

	/**
	 * Generate C include file definitions for element
	 *
	 * @access  public
	 * @return  string C header code 
	 */
	function h_code() 
	{
		return "";
	}

	/**
	 * Generate Editor settings block for C source files
	 *
	 * @access public
	 * @return string Editor settings comment block
	*/
	static function c_code_editor_settings() 
  {
			return '
/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
';
	}


	/**
	 * Generate DocBook documentation code for element
	 *
	 * @access  public
	 * @return  string DocBook XML content
	 */
	abstract function docbook_xml();

	/**
	 * Generate DocBook XML header block for all elements of this class
	 *
	 * @access public
	 * @param  string Extension name
	 * @return string DocBook XML fragment
	 */
	static function docbook_xml_header($name) 
	{
		return "";
	}

	/**
	 * Generate DocBook XML footer block for all elements of this class
	 *
	 * @access public
	 * @param  string Extension name
	 * @return string DocBook XML fragment
	 */
	static function docbook_xml_footer($name) 
	{
		return "";
	}

	/**
	 * Generate Editor settings block for documentation files
	 *
	 * @access public
	 * @param  int    Directory nesting depth of target file (default: 3)
	 * @return string Editor settings comment block
	*/
	static function docbook_editor_settings($level=3) 
  {
		return '
<!-- Keep this comment at the end of the file
Local'.' variables:
mode: sgml
sgml-omittag:t
sgml-shorttag:t
sgml-minimize-attributes:nil
sgml-always-quote-attributes:t
sgml-indent-step:1
sgml-indent-data:t
indent-tabs-mode:nil
sgml-parent-document:nil
sgml-default-dtd-file:"'.str_repeat("../",$level).'manual.ced"
sgml-exposed-tags:nil
sgml-local-catalogs:nil
sgml-local-ecat-files:nil
End:
vim600: syn=xml fen fdm=syntax fdl=2 si
vim: et tw=78 syn=sgml
vi: ts=1 sw=1
-->
';
	}

}

?>