<?php

	/**
	 * Class describing a PHP constant within a PECL extension 
	 *
	 * @access public
	 */
	class PECL_Gen_Element_Constant 
	  extends PECL_Gen_Element 
  {
		/**
		 * The constants name
		 *
		 * @access private
		 * @var    string
		 */
		var $name;

		/**
		 * The constants PHP data type
		 *
		 * @access private
		 * @var    string
		 */
		var $type = "string";

		/**
		 * The constants value
		 *
		 * @access private
		 * @var    string
		 */
		var $value;

		/**
		 * The constants description text
		 *
		 * @access private
		 * @var    string
		 */
		var $desc;


		/**
		 * Set constant name 
		 *
		 * @access public
		 * @param  string  the name
		 * @return bool    true on success
		 */
		function set_name($name)
		{
			if (!self::is_name($name)) {
				return $this->raiseError("'$name'is not a valid constant name");
			}
			
			$this->name = $name;
			
			return true;
		}
		

		/**
		 * Set constant name 
		 *
		 * @access public
		 * @param  string  the type
		 * @return bool    true on success
		 */
		function set_type($type) 
		{
			if (!in_array($type, array('int', 'float', 'string'))) {
				return $this->raiseError("'$type' is not a valid constant type, only int, float and string");
			}														  
			
			$this->type = $type;
			
			return true;
		}

		/**
		 * Set constant value 
		 *
		 * @access public
		 * @param  string  the value
		 * @return bool    true on success
		 */
		function set_value($value)
		{
			$this->value = $value;

			return true;
		}

		/**
		 * Set constant descriptive text
		 *
		 * @access public
		 * @param  string  the name
		 * @return bool    true on success
		 */
		function set_desc($desc)
		{
			$this->desc = $desc;

			return true;
		}

		static function c_code_header($name) {
			return "";
		}
		
		function c_code() {
			switch ($this->type) {
			case "int":
				return "REGISTER_LONG_CONSTANT(\"{$this->name}\", {$this->value}, 0);\n";
			
			case "float":
				return "REGISTER_DOUBLE_CONSTANT(\"{$this->name}\", {$this->value}, 0);\n";

			case "string":
				return "REGISTER_STRING_CONSTANT(\"{$this->name}\", \"$value\", ".strlen($this->value).", 0);\n";
			}
		}

		static function c_code_footer() {
			return "";
		}

		
		static function docbook_xml_header($name) {
			return
"    <table>
     <title>$name constants</title>
      <tgroup cols='3'>
       <thead>
        <row>
         <entry>name</entry>
         <entry>value</entry>
         <entry>descrpition</entry>
        </row>
       </thead>
      <tbody>
";
		}
			
		function docbook_xml() {
			return trim("
<row>
 <entry>
  <constant id='constant".strtolower(str_replace("_","-",$this->name))."'>{$this->name}</constant>
  (<link linkend='language.types.integer'>integer</link>)
 </entry>
 <entry>{$this->value}</entry>
 <entry>{$this->desc}</entry>
</row>
")."\n";
		}

		static function docbook_xml_footer() {
			return
"     </tbody>
    </tgroup>
   </table>
";
		}
	}

?>
