<?php

	/**
	 * Class describing a PHP global within a PECL extension 
	 *
	 * @access public
	 */
	class PECL_Gen_Element_Global 
    extends PECL_Gen_Element 
  {
		/**
		 * The name of the global
		 *
		 * @accesss private
		 * @var     string
		 */
		var $name;

		/**
		 * The type of the global
		 *
		 * @accesss private
		 * @var     string
		 */
		var $type;

		function set_name($name) 
		{
			if (!self::is_name($name)) {
				return $this->raise_error("'$name' is not a valid global name");
			}
			
			$this->name = $name;

			return true;
		}

		function set_type($name)
		{
			if (!self::is_type($name)) {
				return $this->raiseError("'$name' is not a valid type for a global");
			}
			
			$this->type = $type;

			return true;
		}


		/* overriding type check as we deal with C types here
			 check is rather naive as it doesn't know about context
			 so we check for a sequence of valid names for now
			 TODO: check for either simple type, struct/class or single word (typedef)
		*/
		function is_type($type) {
			$array = explode(" ", str_replace('*',' ',$type));
			foreach ($array as $name) {
				if (empty($name)) continue;
				if (!$this->is_name($name)) return false;
			}
			return true;
		}
		


		static function c_code_header($name) {
			return "static void php_{$name}_init_globals(zend_{$name}_globals *{$name}_globals)\n{\n";
		}

		function c_code($name) {
			$code = "  {$name}_globals->{$this->name} = ";
			if (strstr($this->type, "*")) {
				$code .= "NULL;\n";
			} else {
				$code .= "0;\n";
			}
			return $code;
		}

		static function c_code_footer($name) {
			return "
}

static void php_{$name}_shutdown_globals(zend_{$name}_globals *{$name}_globals)
{
}";
		}

		static function h_code_header($name) {
			return "ZEND_BEGIN_MODULE_GLOBALS({$name})\n";
		}

		function h_code($name) {
			return "  {$this->type} {$this->name};\n";
		}

		static function h_code_footer($name) {
			$upname = strtoupper($name);

			return "
ZEND_END_MODULE_GLOBALS({$name})

#ifdef ZTS
#define {$upname}_G(v) TSRMG({$name}_globals_id, zend_{$name}_globals *, v)
#else
#define {$upname}_G(v) ({$name}_globals.v)
#endif

";
			
		}

    function docbook_xml($level=3) 
    {
      // these are real internals, no documentation needed
      return "";
    }


	}

?>
