<?php

	/**
	 * Class describing a PHP ini directive within a PECL extension 
	 *
	 * @access public
	 */
	class PECL_Gen_Element_Ini 
	  extends PECL_Gen_Element 
  {
		/**
		 * Directive name
		 *
		 * @accesss private
		 * @var     string
		 */
		var $name;

		/**
		 * Directive data type
		 *
		 * @accesss private
		 * @var     string
		 */
		var $type;

		/**
		 * Directive default value
		 *
		 * @accesss private
		 * @var     string
		 */
		var $value;

		/**
		 * Directive description
		 *
		 * @accesss private
		 * @var     string
		 */
		var $desc;

		/**
		 * Directive access mode
		 *
		 * @accesss private
		 * @var     string
		 */
		var $access = "PHP_INI_ALL";

		/**
		 * Directive OnUpdate handler
		 *
		 * @accesss private
		 * @var     string
		 */
		var $onupdate;

		/**
		 * Internal C type that stores the directives value
		 *
		 * @accesss private
		 * @var     string
		 */
		var $c_type;


		function __construct($attr) {
			$this->set_type("string");
		}

		function set_name($name)
		{
			if (!self::is_name($name)) {
				return $this->raiseError("'$name' is not a valid php.ini directive name");
			}

			$this->name = $name;

			return true;
		}

		function set_value($value)
		{
			$this->value = value;

			return true;
		}

		function set_desc($desc)
		{
			$this->desc = desc;

			return true;
		}

		function set_access($access)
		{
			switch ($access) {
			case "system":
				$this->access = "PHP_INI_SYSTEM";
				return true;
			case "perdir":
				$this->access = "PHP_INI_PERDIR";
				return true;
			case "user":
				$this->access = "PHP_INI_USER";
				return true;
			case "all":
			case "":
				$this->access = "PHP_INI_ALL";
				return true;
			default:
				return $this->raiseError("'$access' is not a valid access mode (system|perdir|user|all)");
			}
		}

		function set_type($type)
		{
			switch ($type) {
			case "bool":
				$this->onupdate = "OnUpdateBool";
				$this->c_type = "zend_bool";
				return true;					
			case "int":
				$this->onupdate = "OnUpdateLong";
				$this->c_type = "long";
				return true;
			case "float":
				$this->onupdate = "OnUpdateReal";
				$this->c_type = "double";
				return true;
			case "string":
				$this->onupdate = "OnUpdateString";
				$this->c_type = "char *";
				return true;
			default:
				return $this->raiseError("'$this->type' not supported, only bool, int, float and string"); 
			}
		}

		function set_onupdate($name)
		{
			if (!self::is_name($name)) {
				return $this->raiseError("'$name' is not a valid update function name");
			}

			$this->onupdate = $name;

			return true;
		}


		static function c_code_header($name) {
			return "PHP_INI_BEGIN()\n";
		}

		function c_code($name) {
			return "  STD_PHP_INI_ENTRY(\"$name.{$this->name}\", \"{$this->value}\", {$this->access}, {$this->onupdate}, {$this->name}, zend_{$name}_globals, {$name}_globals)\n";
		}

		static function c_code_footer() {
			return "PHP_INI_END()\n\n";
		}



		static function docbook_xml_header($name) {
			return
"    <table>
     <title>$name runtime configuration</title>
			<tgroup cols='3'>
			 <thead>
        <row>
         <entry>directive</entry>
         <entry>default value</entry>
         <entry>descrpition</entry>
        </row>
       </thead>
      <tbody>
";
		}

		function docbook_xml() {
			return 
"    <row>
     <entry>$this->name</entry>
     <entry>$this->value</entry>
     <entry>$this->desc</entry>
    </row>
";
		}

  	static function docbook_xml_footer() {
			return
"     </tbody>
    </tgroup>
   </table>
";
	  }
	}

?>
