<?php
	// {{{ includes

	require_once "System.php";
	
	require_once "PECL/Gen/Element.php";
	require_once "PECL/Gen/Element/Constant.php";
	require_once "PECL/Gen/Element/Function.php";
	require_once "PECL/Gen/Element/Resource.php";
	require_once "PECL/Gen/Element/Ini.php";
	require_once "PECL/Gen/Element/Global.php";
	require_once "PECL/Gen/Element/Logo.php";
	
	require_once "PECL/Gen/Maintainer.php";

	require_once "PECL/Gen/License.php";

	require_once "PECL/Gen/Release.php";

	require_once "PECL/Gen/Tools/Platform.php";

	// }}} 
	
	/**
	 * A class that generates PECL extension soure and documenation files
	 *
	 * @access  public
	 * @version $Id: Extension.php,v 1.8 2003/07/03 11:31:19 hholzgra Exp $
	 */
	
	class PECL_Gen_Extension extends PEAR {
		// {{{ member variables

		/**
		 * The extensions basename (C naming rules apply)
		 *
		 * @var string
		 */
		var $name = "unknown";
		
		
		/**
		 * The extensions descriptive name
		 *
		 * @var string
		 */
		var $summary = "The unknown extension";
		
		/** 
		 * The license for this extension
		 *
		 * @var object
		 */
		var $license  = NULL;
		
		/** 
		 * The release info for this extension
		 *
		 * @var object
		 */
		var $release  = NULL;
		
		/** 
		 * The implementation language
		 *
		 * Currently we support "c" and "cpp"
		 *
		 * @var string
		 */
		var $language  = "c";
		
		/**
		 * The target platform for this extension
		 *
		 * Possible values are "unix", "win" and "all"
		 * 
		 * @var string
		 */
		var	$platform = null;

	
		/**
		 * The authors contributing to this extension
		 *
		 * @var array
		 */
		var	$authors = array();


		/**
		 * The public PHP functions defined by this extension
		 *
		 * @var array
		 */
		var	$functions = array();

		/**
		 * The extensions internal functions like MINIT 
		 *
		 * @var array
		 */
		var	$internal_functions = array();

		/**
		 * The extensions private internal C functions
		 *
		 * @var array
		 */
		var	$private_functions = array();

		/**
		 * The constants defined by this extension
		 *
		 * @var array
		 */
		var	$constants = array();

		/**
		 * The extensions php.ini parameters
		 *
		 * @var array
		 */
		var $phpini    = array();

		/**
		 * The extensions internal global variables
		 *
		 * @var array
		 */
		var $globals    = array();

		/**
		 * The PHP resources defined by this extension
		 *
		 * @var array
		 */
	  var $resources = array();

		/**
		 * The package files created by this extension
		 *
		 * @var array
		 */
		var $package_files = array();

		// }}} 


		// {{{ constructor

		/**
		 * The constructor
		 *
		 * @access public
		 */

		function __construct() {
			// TODO some member specs missing
			$this->streams   = array();
			$this->dependson = array();
			$this->code      = array();
			$this->libs      = array();
			$this->headers   = array();

			$this->release = new PECL_Gen_Release;

			$this->platform = new PECL_Gen_Tools_Platform("all");
		}

		// }}} 
		
		function set_name($name) 
		{
			if (! PECL_Gen_Element::is_name($name)) {
				return $this->raiseError("'$name' is not a valid extension name");
			}

			$this->name = $name;
			return true;
		}

		function set_summary($text) 
		{
			$this->summary = $text;
			return true;
		}

		function set_description($text) 
		{
			$this->description = $text;
			return true;
		}

		function set_language($lang)
		{
			switch (strtolower($lang)) {
			case "c":
				$this->language = "c";
				return true;
			case "cpp":
			case "cxx":
			case "c++":
				$this->language = "cpp";
				return true;
			default:
				break;
			}

			return $this->raiseError("'$lang' is not a supported implementation language");
		}

		function set_platform($type)
		{
			$this->platform = new PECL_Gen_Tools_Platform($type);
			if (PEAR::isError($this->platform)) {
				return $this->platform;
			}

			return true;
		}


		function add_code($type, $code)
		{
			$this->code[$type][] = $code;
		}


		// {{{ member adding functions

  /**
	* Add an author or maintainer to the extension
	*
	* @access public
  * @param  object   a maintainer object
	*/
  function add_author($author)
  {
		if (!is_a($author, "PECL_Gen_Maintainer")) {
			return $this->raiseError("argument is not PECL_Gen_Maintainer");
		}

    $this->authors[$author->user] = $author;

		return true;
  }


  /**
	* Add a function to the extension
	*
	* @access public
  * @param  object   a function object
	*/
  function add_function($function)
  {
		$role = $function->get_role();

		switch ($role) {
		case "public":
			$this->functions[$function->name] = $function;
			return true;
		case "private":
			$this->private_functions[$function->name] = $function;
			return true;
		case "internal":
			$this->internal_functions[$function->name] = $function;
			return true;
		default: 
			return $this->raiseError("unnokwn function role '$role'");
		}
  }


  /**
	* Add a PHP constant to the extension
	*
	* @access public
  * @param  object   a constant object
	*/
  function add_constant($constant)
  {
    $this->constants[$constant->name] = $constant;

		return true;
  }


  /**
	* Add a PHP ini directive
	*
	* @access public
  * @param  object   a phpini object
	*/
  function add_phpini($phpini)
  {
    $this->phpini[$phpini->name] = $phpini;

		return true;
  }


  /**
	* Add a internal global variable
	*
	* @access public
  * @param  object   a global object
	*/
  function add_global($global)
  {
    $this->global[$global->name] = $global;
 
		return true;
	}


  /**
	* Add a PHP resource type
	*
	* @access public
  * @param  object   a resource object
	*/
  function add_resource($resource)
  {
    $this->resources[$resource->name] = $resource;

		return true;
  }


	/**
	 * Add a package file by type and name
	 *
	 * @access public
	 * @param  string  type
	 * @param  string  name
	 */
	function add_package_file($type, $name)
  {
		$this->package_files[$type][$name] = $name;
  }


	/** 
	 * Set phpinfo logo
	 *
	 * @access public
	 * @param  object  the logo
	 */
	function set_logo($logo) 
  {
		$this->logo = $logo;
		
		return true;
  }

	
	/** 
	 * Set release info
	 * 
	 * @access public
	 * @var    object
	 */
	function set_release($release)
  {
		$this->release = $release;

		return true;
  }


	/** 
	 * Set license 
	 * 
	 * @access public
	 * @var    object
	 */
	function set_license($license)
  {
		$this->license = $license;

		return true;
  }
			

// }}} 

		// {{{ output generation
		
		/**
		 * Create the extensions including
		 *
		 * @access public
		 * @param  string Directory to create (default is ./$this->name)
		 */
		function create_extension($dirpath = false, $force = false) 
		{
			// default: create dir in current working directory, 
			// dirname is the extensions base name
			if (empty($dirpath) || $dirpath == ".") {
				$dirpath = "./" . $this->name;
			} 

			// purge and create extension directory
			if (file_exists($dirpath)) {
				if ($force) {
					if (!is_writeable($dirpath) || !@System::rm("-rf $dirpath")) {
						return $this->raiseError("can't purge '$dirpath'");
					}
				} else {
					return $this->raiseError("'$dirpath' already exists, can't create that directory (use '--force' to override)"); 
				}
			}
			if (!@System::mkdir("-p $dirpath")) {
				return $this->raiseError("can't create '$dirpath'");
			}
			
			// make path absolute to be independant of working directory changes
			$dirpath = realpath($dirpath);
		
			echo "Creating '{$this->name}' extension in '$dirpath'\n";

			// generate complete source code
			$this->generate_source($dirpath);

			// generate README file
			 $this->write_readme($dirpath);

			// generate DocBook XML documantation for PHP manual
			$manpath = $dirpath . "/manual/" . str_replace('_', '-', $this->name);
			if (!@System::mkdir("-p $manpath")) {
				return $this->raiseError("can't create '$manpath'", E_USER_ERROR);
			}
			$this->generate_documentation($manpath);			
		}
	
		/**
		 * Create the extensions code soure and project files
		 *
		 * @access public
		 * @param  string Directory to write to
		 */
		function generate_source($dirpath) 
		{
			// generate source and header files
			$this->write_header_file($dirpath);
			$this->write_code_file($dirpath);
			if (isset($this->logo)) {
				$fp = fopen("$dirpath/{$this->name}_logo.h", "w");
				fwrite($fp, $this->logo->h_code());
				fclose($fp);
			}
			
			// generate project files for configure (unices and similar platforms like cygwin)
			if ($this->platform->test("unix")) {
				$this->write_config_m4($dirpath);
			}
			
			// generate project files for Windows platform (VisualStudio/C++ V6)
			if ($this->platform->test("windows")) {
				$this->write_ms_devstudio_dsp($dirpath);
			}
		
			// generate .cvsignore file entries
			$this->write_dot_cvsignore($dirpath);

			// generate EXPERIMENTAL file for unstable release states
			$this->write_experimental($dirpath);
			
			// generate CREDITS file
			$this->write_credits($dirpath);
			
			// generate LICENSE file if license given
			if ($this->license) {
				$this->license->write_license_file("$dirpath/LICENSE");
				$this->files['doc'][] = "LICENSE";
			}

			// generate PEAR/PECL package.xml file
			$this->write_package_xml($dirpath);

			// generate test case templates
			$this->write_test_files($dirpath);
		}
	
		// {{{   docbook documentation

		/**
		 * Create the extension documentation DocBook XML files
		 *
		 * @access public
		 * @param  string Directory to write to
		 */
		function generate_documentation($docdir) {
			$id_name = str_replace('_', '-', $this->name);

			$fp = fopen("$docdir/reference.xml", "w");
			fputs($fp,
"<?xml version='1.0' encoding='iso-8859-1'?>
<!-- ".'$'."Revision: 1.1 $ -->
 <reference id='ref.$id_name'>
  <title>{$this->summary}</title>
  <titleabbrev>$id_name</titleabbrev>

  <partintro>
   <section id='$id_name.intro'>
    &reftitle.intro;
    <para>
{$this->description}
    </para>
   </section>
   
   <section id='$id_name.requirements'>
    &reftitle.required;
    <para>
    </para>
   </section>

   &reference.$id_name.configure;

   <section id='$id_name.resources'>
    &reftitle.resources;
");

  if (empty($this->resources)) {
    fputs($fp, "   &no.resource;\n");
  } else {
    foreach ($this->resources as $resource) {
      fputs($fp, $resource->docbook_xml($this->name));
    }
  }


  fputs($fp,
"   </section>

   <section id='$id_name.constants'>
    &reftitle.constants;
");

  if (empty($this->constants)) {
    fputs($fp, "    &no.constants;\n");
  } else {
    fputs($fp, PECL_Gen_Element_Constant::docbook_xml_header($id_name));

    foreach ($this->constants as $constant) {
      fputs($fp, $constant->docbook_xml($this->name));
    }

    fputs($fp, PECL_Gen_Element_Constant::docbook_xml_footer());
  }
  fputs($fp,
"   </section>
   
  </partintro>

&reference.$id_name.functions;

 </reference>
");
      fputs($fp, PECL_Gen_Element::docbook_editor_settings());

			fclose($fp);

			// configure options and dependencies have their own file
			$fp = fopen("$docdir/configure.xml","w");

      fputs($fp,"\n   <section id='$id_name.requirements'>\n    &reftitle.required;\n");
     	if (empty($this->libs) && empty($this->headers)) {
			  fputs($fp, "    &no.requirement;\n");
		  } else {
        // TODO allow custom text
        if (isset($this->libs)) {
          $libs = array();
          foreach ($this->libs as $lib) {
            $libs[] = $lib['name'];
          }
          $ies = count($libs)>1 ? "ies" :"y";
          fputs($fp, "<para>This extension requires the following librar$ies: ".join(",", $libs)."</para>\n");
        }
        if (isset($this->headers)) {
          $headers = array();
          foreach ($this->headers as $header) {
            $headers[] = $header['name'];
          }
          $s = count($headers)>1 ? "s" : "";
          fputs($fp, "<para>This extension requires the following header$s: ".join(",", $headers)."</para>\n");
        }
		  }
      fputs($fp, "\n   </section>\n\n");

      fputs($fp,"\n   <section id='$id_name.install'>\n    &reftitle.install;\n");
     	if (empty($this->with)) {
			  fputs($fp, "    &no.install;\n");
		  } else {
			  if (isset($this->with['desc'])) {
				  if (strstr($this->with['desc'], "<para>")) {
					  fputs($fp, $this->with['desc']);
					} else {
					  fputs($fp, "    <para>\n".rtrim($this->with['desc'])."\n    </para>\n");
          }
        } else {
				  // TODO default text
        }
		  }
      fputs($fp, "\n   </section>\n\n");

      fputs($fp,"\n   <section id='$id_name.configuration'>\n    &reftitle.runtime;\n");
     	if (empty($this->phpini)) {
			  fputs($fp, "    &no.config;\n");
		  } else {
			  fputs($fp, PECL_Gen_Element_Ini::docbook_xml_header($this->name)); 
			  foreach ($this->phpini as $phpini) {
				  fputs($fp, $phpini->docbook_xml());
			  }
			  fputs($fp, PECL_Gen_Element_Ini::docbook_xml_footer()); 
		  }
      fputs($fp, "\n   </section>\n\n");
			
      fputs($fp, PECL_Gen_Element::docbook_editor_settings());
			fclose($fp);

			@mkdir("$docdir/functions");
			foreach ($this->functions as $name => $function) {
				$filename = $docdir . "/functions/" . strtolower(str_replace("_", "-", $name)) . ".xml";
				$funcfile = fopen($filename, "w");
				fputs($funcfile, $function->docbook_xml());
				fclose($funcfile);
			}
		}

		// }}} 




	  // {{{   extension entry
		/**
		 * Create the module entry code for this extension
		 *
		 * @access private
     * @return string  zend_module_entry code fragment
		 */
		function generate_extension_entry() {
      $name = $this->name;
      $upname = strtoupper($this->name);

			$code = "
/* {{{ {$name}_module_entry
 */
zend_module_entry {$name}_module_entry = {
	STANDARD_MODULE_HEADER,
	\"$name\",
	{$name}_functions,
	PHP_MINIT($name),     /* Replace with NULL if there is nothing to do at php startup   */ 
	PHP_MSHUTDOWN($name), /* Replace with NULL if there is nothing to do at php shutdown  */
	PHP_RINIT($name),		  /* Replace with NULL if there is nothing to do at request start */
	PHP_RSHUTDOWN($name),	/* Replace with NULL if there is nothing to do at request end   */
	PHP_MINFO($name),
	\"".$this->release->version."\", 
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

";

      $code .= "#ifdef COMPILE_DL_$upname\n";
	    if ($this->language == "cpp") $code .= "extern \"C\" {\n";
      $code .= "ZEND_GET_MODULE($name)\n";
	    if ($this->language == "cpp") $code .= "}\n";
	    $code .= "#endif\n\n";

			return $code;
		}

	// }}} 

	  // {{{ globals and ini
		/**
		 * Create the module globals c code fragment
		 *
		 * @access private
     * @return string  module globals code fragment
		 */

	  function generate_globals_c() {
			if (empty($this->globals)) return "";
			
			$code = "ZEND_DECLARE_MODULE_GLOBALS({$this->name})\n\n";
			
			if (!empty($this->phpini)) {
				$code .= PECL_Gen_Element_Ini::c_code_header($this->name);
				foreach ($this->phpini as $phpini) {
          $code .= $phpini->c_code($this->name);
				}
				$code .= PECL_Gen_Element_Ini::c_code_footer();
			}

			if (!empty($this->globals)) {
				$code .= PECL_Gen_Element_Global::c_code_header($this->name);
				foreach ($this->globals as $global) {
          $code .= $global->c_code($this->name);
				}
				$code .= PECL_Gen_Element_Global::c_code_footer($this->name);
			}

			return $code;
		}



		/**
		 * Create the module globals c header file fragment
		 *
		 * @access private
     * @return string  module globals code fragment
		 */
	  function generate_globals_h() {
			if (empty($this->globals)) return "";
			
			$code = PECL_Gen_Element_Global::h_code_header($this->name);
			foreach ($this->globals as $global) {
        $code .= $global->h_code($this->name);
			}
			$code .= PECL_Gen_Element_Global::h_code_footer($this->name);
			
			return $code;
		}

	// }}} 

	// {{{ license and authoers
		/**
		 * Create the license part of the source file header comment
		 *
		 * @access private
     * @return string  code fragment
		 */
	  function get_license() {

			$code = "/*\n";
			$code.= "   +----------------------------------------------------------------------+\n";

      if (is_object($this->license)) {
        $code.= $this->license->license_comment();
      } else {
        $code.= sprintf("   | unkown license: %-52s |\n", $this->license);
			}

			$code.= "   +----------------------------------------------------------------------+\n";

			foreach ($this->authors as $author) {
				$code.= $author->comment();
			}

			$code.= "   +----------------------------------------------------------------------+\n";
			$code.= "*/\n\n";

			$code.= "/* $ Id: $ */ \n\n";

			return $code;
	  }

	// }}} 


	// {{{ header file

		/**
		 * Write the complete C header file
		 *
		 * @access private
     * @param  string  directory to write to
		 */
	  function write_header_file($dirpath) {
			$filename = "php_{$this->name}.h";

			$this->add_package_file("h", $filename); 
			$fp = fopen("$dirpath/$filename", "w");

			$upname = strtoupper($this->name);
			
			fputs($fp, $this->get_license());
			fputs($fp, "#ifndef PHP_{$upname}_H\n");
			fputs($fp, "#define PHP_{$upname}_H\n\n");

			if (isset($this->headers)) {
				foreach ($this->headers as $header) {
					if (@$header["prepend"] === "yes") {
						fputs($fp, "#include <$header[name]>\n");
					}
				}
			}

			if ($this->language == "cpp") {
				fputs($fp, "extern \"C\" {\n");
			}

			fputs($fp, '
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_'.$upname.'

#include <php.h>
#include <php_ini.h>
#include <SAPI.h>
#include <ext/standard/info.h>

');

			if ($this->language == "cpp") {
				fputs($fp, "}\n");
			}

			if (isset($this->headers)) {
				foreach ($this->headers as $header) {
					if (@$header["prepend"] !== "yes") {
						fputs($fp, "#include <$header[name]>\n");
					}
				}
			}

			if (isset($this->code["header"])) {
				foreach ($this->code["header"] as $code) {
					fputs($fp, $code);
				}
			}

			fputs($fp, "
extern zend_module_entry {$this->name}_module_entry;
#define phpext_{$this->name}_ptr &{$this->name}_module_entry

#ifdef PHP_WIN32
#define PHP_{$upname}_API __declspec(dllexport)
#else
#define PHP_{$upname}_API
#endif

PHP_MINIT_FUNCTION({$this->name});
PHP_MSHUTDOWN_FUNCTION({$this->name});
PHP_RINIT_FUNCTION({$this->name});
PHP_RSHUTDOWN_FUNCTION({$this->name});
PHP_MINFO_FUNCTION({$this->name});

#ifdef ZTS
#include \"TSRM.h\"
#endif

");

			fputs($fp, $this->generate_globals_h());

			fputs($fp, "\n");

			foreach ($this->functions as $name => $function) {
				fputs($fp, "PHP_FUNCTION($name);\n");
			}
			
			fputs($fp, "\n");

			fputs($fp, "#endif /* PHP_HAVE_{$upname} */\n\n");
			fputs($fp, "#endif /* PHP_{$upname}_H */\n\n");

			fputs($fp, PECL_Gen_Element::c_code_editor_settings());

			fclose($fp);
		}

	// }}} 

	// {{{ internal functions

		/**
		 * Create code for the internal functions like MINIT etc ...
		 *
		 * @access private
     * @return string  code snippet
		 */

		function internal_functions_c() {
			$need_block = false;

			$code = "
/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION({$this->name})
{
";

			if (count($this->globals)) {
				$code .= "\tZEND_INIT_MODULE_GLOBALS({$this->name}, php_{$this->name}_init_globals, php_{$this->name}_shutdown_globals)\n";
				$need_block = true;
			}

			if (count($this->phpini)) {
				$code .= "\tREGISTER_INI_ENTRIES();\n";
				$need_block = true;
			}
			
			if (isset($this->logo)) {
				$code .= $this->logo->minit_code();
				$need_block = true;
			}
			
			if (count($this->constants)) {
				foreach ($this->constants as $constant) {
					$code .= $constant->c_code();
				}
				$need_block = true;
			}
			
			if (count($this->resources)) {
				foreach ($this->resources as $resource) {
					$code .= $resource->minit_code();
				}
				$need_block = true;			
			}
			
			if (isset($this->internal_functions['MINIT'])) {
				if ($need_block) $code .= "\n\t{\n";
				$code .= $this->internal_functions['MINIT']->code;
				if ($need_block) $code .= "\n\t}\n";
			} else {
				$code .="\n\t/* add your stuff here */\n";
			}
			$code .= "
  return SUCCESS;
}
/* }}} */

";
			
			$code .= "
/* {{{ PHP_MSHUTDOWN_FUNCTION */
PHP_MSHUTDOWN_FUNCTION({$this->name})
{
";
			
			if (count($this->phpini)) {
			  $code .= "\tUNREGISTER_INI_ENTRIES();\n";
		  }

		  if (isset($this->internal_functions['MSHUTDOWN'])) {
        if (count($this->phpini)) $code .= "\n\t{\n";
			  $code .= $this->internal_functions['MSHUTDOWN']->code;
        if (count($this->phpini)) $code .= "\n\t}\n";
		  } else {
		    $code .="\n\t/* add your stuff here */\n";
      }

		  $code .= "
  
  return SUCCESS;
}
/* }}} */

";

	  	$code .= "
/* {{{ PHP_RINIT_FUNCTION */
PHP_RINIT_FUNCTION({$this->name})
{
";

		  if (isset($this->internal_functions['RINIT'])) {
			  $code .= $this->internal_functions['RINIT']->code;
		  } else {
         $code .= "  /* add your stuff here */\n";
      }

      $code .= "
\treturn SUCCESS;
}
/* }}} */

";

		  $code .= "
/* {{{ PHP_RSHUTDOWN_FUNCTION */
PHP_RSHUTDOWN_FUNCTION({$this->name})
{
";

		  if (isset($this->internal_functions['RSHUTDOWN'])) {
			  $code .= $this->internal_functions['RSHUTDOWN']->code;
		  } else {
         $code .= "  /* add your stuff here */\n";
      }

      $code .= "
\treturn SUCCESS;
}
/* }}} */

";
	
      $code .= "
/* {{{ PHP_MINFO_FUNCTION */
PHP_MINFO_FUNCTION({$this->name})
{
	php_info_print_box_start(0);
";

   if ($this->logo)
   {
     $code.= "
	php_printf(\"<img src='\");
	if (SG(request_info).request_uri) {
		php_printf(\"%s\", (SG(request_info).request_uri));
	}	
	php_printf(\"?=%s\", ".($this->logo->id).");
	php_printf(\"' align={'right' alt='image' border='0'>\\n\");

"; 
    }

   if (isset($this->summary)) {
     $code .= "  php_printf(\"<p>".str_replace('"','\\"',$this->summary)."</p>\\n\");\n";
   }
   if (isset($this->release)) {
     $code .= "  php_printf(\"<p>Version {$this->release->version}{$this->release->state} ({$this->release->date})</p>\\n\");\n";
   }

   if (count($this->authors)) {
     $code .= "  php_printf(\"<p><b>Authors:</b></p>\\n\");\n";
     foreach ($this->authors as $author) {
       $code.= $author->phpinfo();
     }
   }

    $code.=
"	php_info_print_box_end();
";

		if (isset($this->internal_functions['MINFO'])) {
      $code .= "\n\t{\n";
			$code .= $this->internal_functions['MINFO']->code;
      $code .= "\n\t}\n";
		} else {
       $code .= "\t/* add your stuff here */\n";
    }


if (count($this->phpini)) {
	$code .= "\n\tDISPLAY_INI_ENTRIES();";
}
$code .= "
}
/* }}} */

";

		return $code;
  }

	// }}} 

		/**
		 * Create code for private helper functions
		 *
		 * @access private
     * @return string  code snippet
		 */

	function private_functions_c() {
		$code = "";

		foreach ($this->private_functions as $name => $func) {
      $code .= "\n\t/* {{{ $name() */\n{$func->code}\n\t/* }}} */\n\n";
    }

  	return $code;
  }

	// {{{ public functions
		/**
		 * Create code for the exported PHP functions
		 *
		 * @access private
     * @return string  code snippet
		 */
	function public_functions_c() {
		$code = "";

    foreach ($this->functions as $function) {
      $code .= $function->c_code(&$this);
    }
		
		return $code;
	}

	// }}} 


  // {{{ code file

    /**
		 * Write the complete C code file
		 *
		 * @access private
     * @param  string  directory to write to
		 */
	function write_code_file($dirpath) {
      $filename = "{$this->name}.{$this->language}";  // todo extension logic

			$this->add_package_file("c", $filename); 
			$fp = fopen("$dirpath/$filename", "w");

			$upname = strtoupper($this->name);
			
			fputs($fp, $this->get_license());

			fputs($fp, "#include \"php_{$this->name}.h\"\n\n");
						
      if (isset($this->logo)) {
			  fputs($fp, $this->logo->c_code());	
      }

			if (!empty($this->globals)) {
				fputs($fp, "ZEND_DECLARE_MODULE_GLOBALS({$this->name})\n\n");
			}

			fputs($fp, $this->private_functions_c());

			if (!empty($this->resources)) {
				foreach ($this->resources as $resource) {
					fputs($fp, $resource->c_code());
				}
			}

			fputs($fp, "/* {{{ {$this->name}_functions[] */\n");
			fputs($fp, "function_entry {$this->name}_functions[] = {\n");
			foreach ($this->functions as $name => $function) {
				fputs($fp, sprintf("\tPHP_FE(%-20s, NULL)\n",$name));
			}
			fputs($fp, "\t{ NULL, NULL, NULL }\n");
			fputs($fp, "};\n/* }}} */\n\n");
			
			fputs($fp, $this->generate_extension_entry());

			fputs($fp, "\n/* {{{ globals and ini entries */\n");
			fputs($fp, $this->generate_globals_c());
			fputs($fp, "/* }}} */\n\n");

			fputs($fp, $this->internal_functions_c());

			fputs($fp, $this->public_functions_c());

			fputs($fp, PECL_Gen_Element::c_code_editor_settings());
	}

	// }}} 


	// {{{ config.m4 file

    /**
		 * Write config.m4 file for autoconf 
		 *
		 * @access private
     * @param  string  directory to write to
		 */
	function write_config_m4($dirpath) {
		$upname = strtoupper($this->name);
		
		ob_start();

		echo 
'dnl
dnl $ Id: $
dnl
';
		
		if (isset($this->with['attr'])) {
			echo " 
PHP_ARG_WITH({$this->name}, whether to enable {$this->name} functions,
[  --with-{$this->name}[=DIR]      With {$this->name} support], yes)
";

		echo "
if test \"\$PHP_$upname\" != \"no\"; then
  if test -r \"\$PHP_$upname/{$this->with['attr']['testfile']}\"; then
    PHP_{$upname}_DIR=\"\$PHP_$upname\"
  else
    AC_MSG_CHECKING(for {$this->name} in default path)
    for i in ".str_replace(":"," ",$this->with['attr']['defaults'])."; do
      if test -r \"\$i/{$this->with['attr']['testfile']}\"; then
        PHP_{$upname}_DIR=\$i
        AC_MSG_RESULT(found in \$i)
      fi
    done
  fi
fi

PHP_ADD_INCLUDE(\$PHP_{$upname}_DIR/include)
";

		} else {
		  echo "
PHP_ARG_ENABLE({$this->name} , whether to enable {$this->name} functions,
[  --disable-{$this->name}         Disable {$this->name} support], yes)
";
		}

    echo "\n";


		if (count($this->libs)) {
      $first = true;

			foreach ($this->libs as $lib) {
				if (!$lib["platform"]->test("unix")) {
					continue;
				}

        if ($first) {
          echo "PHP_SUBST({$upname}_SHARED_LIBADD)\n\n";
          $first = false;
        }

		    if (isset($this->with['attr'])) {
          echo "PHP_ADD_LIBRARY_WITH_PATH($lib[name], \$PHP_{$upname}_DIR/lib, {$upname}_SHARED_LIBADD)\n";
        } else {
          echo "PHP_ADD_LIBRARY($lib[name],, {$upname}_SHARED_LIBADD)\n";
        }

			  if (isset($lib['function'])) {
           echo "AC_CHECK_LIB($lib[name], $lib[function], [AC_DEFINE(HAVE_".strtoupper($lib['name']).",1,[ ])], [AC_MSG_ERROR($lib[name] library not found or wrong version)],)\n";
        }
			}
    }

    echo "\n";

		if ($this->language === "cpp") {
			echo "PHP_REQUIRE_CXX\n";
			echo "PHP_ADD_LIBRARY(stdc++)\n";
		}

		echo "
if test \"\$PHP_$upname\" != \"no\"; then
  AC_DEFINE(HAVE_$upname, 1, [ ])
  PHP_NEW_EXTENSION({$this->name}, ".join(" ", $this->package_files['c'])." , \$ext_shared)
fi

";

		$filename = "$dirpath/config.m4";
		$this->add_package_file("conf", "config.m4");

		$fp = fopen($filename, "w");
		fputs($fp, ob_get_contents());
		fclose($fp);
    ob_end_clean();
	}

	// }}} 

	// {{{ M$ dev studio project file
    /**
		 * Write project file for VisualStudio V6
		 *
		 * @access private
     * @param  string  directory to write to
		 */
	function write_ms_devstudio_dsp($dirpath) {
    ob_start();

    // these system libraries are always needed?
    // (list taken from sample *.dsp files in php ext tree...)
    $winlibs = "kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib advapi32.lib ";
    $winlibs.= "shell32.lib ole32.lib oleaut32.lib uuid.lib odbc32.lib odbccp32.lib";

    // add libraries from <deps> section
    if (count($this->libs)) {
      foreach ($this->libs as $lib) {
				if (!$lib["platform"]->test("windows")) {
					continue;
				}
        $winlibs .= " $lib[name].lib";
      }
    }

		echo
'# Microsoft Developer Studio Project File - Name="'.$this->name.'" - Package Owner=<4>
# Microsoft Developer Studio Generated Build File, Format Version 6.00
# ** DO NOT EDIT **

# TARGTYPE "Win32 (x86) Dynamic-Link Library" 0x0102

CFG='.$this->name.' - Win32 Debug_TS
!MESSAGE This is not a valid makefile. To build this project using NMAKE,
!MESSAGE use the Export Makefile command and run
!MESSAGE 
!MESSAGE NMAKE /f "'.$this->name.'.mak".
!MESSAGE 
!MESSAGE You can specify a configuration when running NMAKE
!MESSAGE by defining the macro CFG on the command line. For example:
!MESSAGE 
!MESSAGE NMAKE /f "'.$this->name.'.mak" CFG="'.$this->name.' - Win32 Debug_TS"
!MESSAGE 
!MESSAGE Possible choices for configuration are:
!MESSAGE 
!MESSAGE "'.$this->name.' - Win32 Release_TS" (based on "Win32 (x86) Dynamic-Link Library")
!MESSAGE "'.$this->name.' - Win32 Debug_TS" (based on "Win32 (x86) Dynamic-Link Library")
!MESSAGE 

# Begin Project
# PROP AllowPerConfigDependencies 0
# PROP Scc_ProjName ""
# PROP Scc_LocalPath ""
CPP=cl.exe
MTL=midl.exe
RSC=rc.exe

!IF  "$(CFG)" == "'.$this->name.' - Win32 Release_TS"

# PROP BASE Use_MFC 0
# PROP BASE Use_Debug_Libraries 0
# PROP BASE Output_Dir "Release_TS"
# PROP BASE Intermediate_Dir "Release_TS"
# PROP BASE Target_Dir ""
# PROP Use_MFC 0
# PROP Use_Debug_Libraries 0
# PROP Output_Dir "Release_TS"
# PROP Intermediate_Dir "Release_TS"
# PROP Ignore_Export_Lib 0
# PROP Target_Dir ""
# ADD BASE CPP /nologo /MT /W3 /GX /O2 /D "WIN32" /D "NDEBUG" /D "_WINDOWS" /D "_MBCS" /D "_USRDLL" /D "'.strtoupper($this->name).'_EXPORTS" /YX /FD /c
# ADD CPP /nologo /MT /W3 /GX /O2 /I "..\.." /I "..\..\Zend" /I "..\..\TSRM" /I "..\..\main" /D "WIN32" /D "PHP_EXPORTS" /D "COMPILE_DL_'.strtoupper($this->name).'" /D ZTS=1 /D HAVE_'.strtoupper($this->name).'=1 /D ZEND_DEBUG=0 /D "NDEBUG" /D "_WINDOWS" /D "ZEND_WIN32" /D "PHP_WIN32" /YX /FD /c
# ADD BASE MTL /nologo /D "NDEBUG" /mktyplib203 /win32
# ADD MTL /nologo /D "NDEBUG" /mktyplib203 /win32
# ADD BASE RSC /l 0x407 /d "NDEBUG"
# ADD RSC /l 0x407 /d "NDEBUG"
BSC32=bscmake.exe
# ADD BASE BSC32 /nologo
# ADD BSC32 /nologo
LINK32=link.exe
# ADD BASE LINK32 '.$winlibs.' /nologo /dll /machine:I386
# ADD LINK32 php4ts.lib '.$winlibs.' /nologo /dll /machine:I386 /out:"..\..\Release_TS\php_'.$this->name.'.dll" /libpath:"..\..\Release_TS" /libpath:"..\..\Release_TS_Inline"

!ELSEIF  "$(CFG)" == "'.$this->name.' - Win32 Debug_TS"

# PROP BASE Use_MFC 0
# PROP BASE Use_Debug_Libraries 1
# PROP BASE Output_Dir "Debug_TS"
# PROP BASE Intermediate_Dir "Debug_TS"
# PROP BASE Target_Dir ""
# PROP Use_MFC 0
# PROP Use_Debug_Libraries 1
# PROP Output_Dir "Debug_TS"
# PROP Intermediate_Dir "Debug_TS"
# PROP Ignore_Export_Lib 0
# PROP Target_Dir ""
# ADD BASE CPP /nologo /MTd /W3 /Gm /GX /ZI /Od /D "WIN32" /D "_DEBUG" /D "_WINDOWS" /D "_MBCS" /D "_USRDLL" /D "'.strtoupper($this->name).'_EXPORTS" /YX /FD /GZ  /c
# ADD CPP /nologo /MTd /W3 /Gm /GX /ZI /Od /I "..\.." /I "..\..\Zend" /I "..\..\TSRM" /I "..\..\main" /D ZEND_DEBUG=1 /D "WIN32" /D "_DEBUG" /D "_WINDOWS" /D "PHP_EXPORTS" /D "COMPILE_DL_'.strtoupper($this->name).'" /D ZTS=1 /D "ZEND_WIN32" /D "PHP_WIN32" /D HAVE_'.strtoupper($this->name).'=1 /YX /FD /GZ  /c
# ADD BASE MTL /nologo /D "_DEBUG" /mktyplib203 /win32
# ADD MTL /nologo /D "_DEBUG" /mktyplib203 /win32
# ADD BASE RSC /l 0x407 /d "_DEBUG"
# ADD RSC /l 0x407 /d "_DEBUG"
BSC32=bscmake.exe
# ADD BASE BSC32 /nologo
# ADD BSC32 /nologo
LINK32=link.exe
# ADD BASE LINK32 '.$winlibs.' /nologo /dll /debug /machine:I386 /pdbtype:sept
# ADD LINK32 php4ts_debug.lib '.$winlibs.' /nologo /dll /debug /machine:I386 /out:"..\..\Debug_TS\php_'.$this->name.'.dll" /pdbtype:sept /libpath:"..\..\Debug_TS"

!ENDIF 

# Begin Target

# Name "'.$this->name.' - Win32 Release_TS"
# Name "'.$this->name.' - Win32 Debug_TS"
';


		echo '
# Begin Group "Source Files"

# PROP Default_Filter "cpp;c;cxx;rc;def;r;odl;idl;hpj;bat"
';

		foreach ($this->package_files['c'] as $filename) {
			if ($filename{0}!='/' && $filename{0}!='.') {
				$filename = "./$filename";
			}
			$filename = str_replace("/","\\", $filename);

		 echo "
# Begin Source File

SOURCE=$filename
# End Source File
";
		}

echo '
# End Group
';




echo '
# Begin Group "Header Files"

# PROP Default_Filter "h;hpp;hxx;hm;inl"
';

		foreach ($this->package_files['h'] as $filename) {
			if ($filename{0}!='/' && $filename{0}!='.') {
				$filename = "./$filename";
			}
			$filename = str_replace("/","\\",$filename);

			echo "
# Begin Source File

SOURCE=$filename
# End Source File
";
		}

echo
'# End Group
# End Target
# End Project
';

    $filename = "$dirpath/{$this->name}.dsp"; 
		$this->add_package_file("conf", $this->name.".dsp");

    // write file, enforce DOS/Windows line endings on all platforms
		$fp = fopen($filename,"wb");
    fputs($fp, str_replace("\n","\r\n",ob_get_contents()));
    fclose($fp);
    ob_end_clean();
	}

// }}} 





    /**
		 * Write authors to the CREDITS file
		 *
		 * @access private
     * @param  string  directory to write to
		 */
	function write_credits($dirpath) {
	  if (count($this->authors)) {
       $this->add_package_file("doc", "CREDITS");
       $fp = fopen("$dirpath/CREDITS", "w");
       fputs($fp, "{$this->name}\n");
       $names = array();
       foreach($this->authors as $author) {
         $names[] = $author->name;
       }
       fputs($fp, join(", ", $names) . "\n"); 
       fclose($fp);
    }
  }


    /**
		 * Write EXPERIMENTAL file for non-stable extensions
		 *
		 * @access private
     * @param  string  directory to write to
		 */
  function write_experimental($dirpath) {
    if (($this->release) && isset($this->release->state) && $this->release->state !== 'stable') {
      $this->add_package_file("doc", "EXPERIMENTAL");
      $fp = fopen("$dirpath/EXPERIMENTAL", "w");
      fputs($fp,
"this extension is experimental,
its functions may change their names 
or move to extension all together 
so do not rely to much on them 
you have been warned!
");
      fclose($fp);
    }
  }

    /**
		 * Write PEAR/PECL package.xml file
		 *
		 * @access private
     * @param  string  directory to write to
		 */

	function write_package_xml($dirpath) {
		$status = false;

		ob_start();
		
		echo 
"<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>
<!DOCTYPE package SYSTEM \"http://pear.php.net/package.dtd\">
<package>
  <name>{$this->name}</name>
";

		if (isset($this->summary)) {
			echo "  <summary>{$this->summary}</summary>\n";
		}

		if (isset($this->description)) {
			echo "  <description>\n".rtrim($this->description)."\n  </description>\n";
		}
		
    if ($this->license) {
      echo "  <license>".$this->license."</license>\n";
    }

		if (isset($this->with['attr'])) {
      echo "  <configureoptions>\n";
      echo "   <configureoption name=\"{$this->name}\" default=\"shared\" prompt=\"{$this->name} installation directory?\" />\n";
      echo "  </configureoptions>\n";
    }

		if (count($this->authors)) {
			echo "\n  <maintainers>\n";
			foreach ($this->authors as $author) {
        echo $author->package_xml();
			}
			echo "  </maintainers>\n";
		}
		
		if (isset($this->release)) {
      echo $this->release->package_xml();
		}

		echo "\n  <filelist>\n";
		echo "    <dir role=\"doc\" name=\"/\">\n";
		if (@is_array($this->package_files['doc'])) {
			foreach ($this->package_files['doc'] as $file) {
				echo "      <file role=\"doc\">$file</file>\n";
			}
		}

    foreach (array("conf", "c", "h") as $type) { 
			foreach ($this->package_files[$type] as $file) {
			  echo "      <file role=\"src\">$file</file>\n";
		  }
		}
		
		echo "    </dir>\n";
		echo "  </filelist>\n";


		echo "</package>\n";
		
		$fp = fopen("$dirpath/package.xml", "w");
		if (is_resource($fp)) {
			fputs($fp, ob_get_contents());
			fclose($fp);
		}
		
		ob_end_clean();
		return $status;
	}

	  // }}}

  function write_test_files() {
    @mkdir("$this->name/tests");
    
    foreach ($this->functions as $function) {
      $function->write_test($this);
    }
  }

  /**
  * Write .cvsignore entries
  *
  * @access public
  * @param  string  directory to write to
  */
  function write_dot_cvsignore($dirpath)
  {
    // open output file
    $fp = fopen("$dirpath/.cvsignore", "w");

    // unix specific entries
		if ($this->platform->test("unix")) {
      fputs($fp, "*.lo\n");
      fputs($fp, "*.la\n");
      fputs($fp, ".deps\n");
    }

    // windows specific entries
		if ($this->platform->test("windows")) {
      fputs($fp, "*.plg\n");
      fputs($fp, "*.opt\n");
	    fputs($fp, "*.ncb\n");
			fputs($fp, "Release\n");
      fputs($fp, "Release_inline\n");
      fputs($fp, "Debug\n");
      fputs($fp, "Release_TS\n");
      fputs($fp, "Release_TSDbg\n");
      fputs($fp, "Release_TS_inline\n");
      fputs($fp, "Debug_TS\n");
    }

    fclose($fp);
  }

  /**
  * Describe next steps after successfull extension creation
  *
  * @access private
  * @param  string  directory where extension was build
  */
  function success_msg($dirpath =false)
  {

    if ($dirpath === false) {
      $dirpath = $this->name;
    }

    $msg = "Your extension has been created in directory $dirpath.\n";
    if (!isset($this->readme)) {
       $msg.= "See $dirpath/README for further instructions.\n";
    }

    return $msg;
  }


  /** 
  * Generate README file (custom or default)
  *
  * @access private
  * @param  string  directory to write to
  */
  function write_readme($dirpath) 
  {
    $config_option = isset($this->with['attr']) ? "--with" : "--enable";
    $config_option.= $this->name;

		ob_start();
?>
This is a standalone PHP extension created using PECL_Gen <?php echo self::version(); ?>


HACKING
=======

There are two ways to modify an extension created using PECL_Gen:

1) you can modify the generated code as with any other PHP extension
  
2) you can add custom code to the PECL_Gen XML source and re-run pecl-gen

The 2nd approach may look a bit complicated but you have be aware that any
manual changes to the generated code will be lost if you ever change the
XML specs and re-run PECL-Gen. All changes done before have to be applied
to the newly generated code again.
Adding code snippets to the XML source itself on the other hand may be a 
bit more complicated but this way your custom code will always be in the
generated code no matter how often you rerun PECL_Gen.

<?php if ($this->platform->test("unix")): ?>

BUILDING ON UNIX etc.
=====================

To compile your new extension, you will have to execute the following steps:

1.  $ ./phpize
2.  $ ./configure [<?php echo $config_option; ?>]
3.  $ make
[4. $ make test ] # NOTE: this doesn't work right now *)
5.  $ [sudo] make install

*) this is a general problem with "make test" and standalone extensions
   (that is being worked on) so please don't blame PECL_Gen for this
   
<?php endif; ?>

<?php if ($this->platform->test("windows")): ?>

BUILDING ON WINDOWS
===================

The extension provides the VisualStudio V6 project file 

  <?php echo $this->extname.".dsp" ?>

To compile the extension you open this file using VisualStudio,
select the apropriate configuration for your installation
(either "Release_TS" or "Debug_TS") and create "php_<?php echo $this->name; ?>.dll"

After successfull compilation you have to copy the newly
created "php_<?php echo $this->name; ?>.dll" to the PHP
extension directory (default: C:\PHP\extensions).

<?php endif; ?>

TESTING
=======

You can now load the extension using a php.ini directive

  extension="php_<?php echo $this->name; ?>.[so|dll]"

or load it at runtime using the dl() function

  dl("php_<?php echo $this->name; ?>.[so|dll]");

The extension should now be available, you can test this
using the extension_loaded() function:

  if (extension_loaded(<?php echo $this->name; ?>))
    echo "<?php echo $this->name; ?> loaded :)";
  else
    echo "something is wrong :(";

The extension will also add its own block to the output
of phpinfo();

<?php


    $fp = fopen("$dirpath/README", "w");
    fputs($fp, ob_get_contents());
    ob_end_clean();
    fclose($fp);
  }


  /**
  * Current version number
  *
  * @access public
  * @return string version
  */
  static function version() 
  {
    return "0.7.0";
  }
}	

?>