<?php

/**
* Abstract base class for license handling
* 
* @version $Id: License.php,v 1.2 2003/06/30 10:03:41 hholzgra Exp $
*/

abstract class PECL_Gen_License extends PEAR
{
	/**
	 * Constructor
	 *
	 * @access private
	 * @param  License options
	 */
	function __construct($options = array()) {
		$this->options = $options;
	}

	/**
	 * Takes a License shortname and returns an instantiated object of that license
	 *
	 * @access  public
	 * @param   string  License shortname, e.g. PHP, BSD, LGPL
	 * @param   array   License options
	 * @returns object  License instance
	 */
	static function factory($name, $options=array()) {
		$classname = "PECL_Gen_License_".strtoupper($name);
		$classfile = str_replace("_", "/", $classname).".php";

		if (!class_exists($classname)) {
			if (!@include_once "$classfile")
				$this->raiseError("Unknown license type '$name' ", E_USER_WARNING);
		}

		return 
			class_exists($classname) 
			? new $classname($options) 
			: false;
	}

	/**
	 * Writes the License text to a file
	 *
	 * @param   string Filename to write to (default is ./LICENSE) 
	 * @return  bool   Success state
	 * @access  public
	 */
	function write_license_file($path = "./LICENSE") {
		$fp = fopen($path, "w");

		if (is_resource($fp)) {
			fputs($fp, $this->license_file_text()); 
			fclose($fp);
			return true;
		}

		return false;
	}
	
	/**
	 * Returns a string suitable for insertion into comment headers
	 * 
	 * @return string    License comment
	 * @access public
	 */
	abstract function license_comment();

	/**
	 * Returns the complete license text as string
	 * 
	 * @return string    License text
	 * @access public
	 */
	abstract function license_file_text();
}


?>