<?php

/**
 * A class that describes an extension author or maintainer
 *
 * @access public
 * @version $Id: Maintainer.php,v 1.1 2003/06/29 12:24:54 hholzgra Exp $
 */
class PECL_Gen_Maintainer extends PEAR
{
	/**
	 * CVS user name
	 *
	 * @var string
	*/
	var $user;

	/**
	 * Real name
	 *
	 * @var string
	 */
	var $name;

	/**
	 * Email address
	 *
	 * @var string
	 */
	var $email;

	/**
	 * Role in this project
	 *
	 * @var string
	 */
	 var $role;
	 
	/**
	 * First maintainer added?
	 *
	 * @var bool
	 */
	 static $first = true;

	/**
	 * Prefix to use in comment headers
	 *
	 * @var bool
	 */
	var  $comment_prefix = "Authors:";

	 /**
	 * Constructor
	 *
	 * @access public
	 * @param  string  CVS user name
	 * @param  string  real name
	 * @param  string  email address
	 * @param  string  role in this project
	 */
	 function __construct($user="unknown", $name="Anonymous Coward", $email="unknown", $role="unknown")
	 {
	   $this->user  = $user;
		 $this->name  = $name;
		 $this->email = $email;
		 $this->role  = $role;

		 if (self::$first) {
			 self::$first = false;
		 } else {
			 $this->comment_prefix = "        ";
		 }
	 }

	 /**
		* Set CVS user name
		*
		* @access public
		* @param  string CVS user name
		* @return bool   true on success
		*/
	 function set_user($name) 
	 {
		 if (!ereg("^[[:alnum:]_-]+$", $name)) {
			 return $this->raiseError("'$name' is not a valid CVS user name");
		 }

		 $this->user = $name;
		 return true;
	 }

	 /**
		* Set real user name
		*
		* @access public
		* @param  string user name
		* @return bool   true on success
		*/
	 function set_name($name) 
	 {
		 $this->name = $name;
		 return true;
	 }

	 /**
		* Set email address
		*
		* @access public
		* @param  string email address
		* @return bool   true on success
		*/
	 function set_email($email) 
	 {
		 // TODO check for valid address

		 $this->email = $email;
		 return true;
	 }

	 /**
		* Set project role
		*
		* @access public
		* @param  string project role
		* @return bool   true on success
		*/
	 function set_role($role) 
	 {
		 // TODO check for valid roles

		 $this->role = $role;
		 return true;
	 }

	 /**
		* Generate a comment header line for this author
		*
		* @access public
		* @return string comment line 
		*/
	 function comment()
	 {
		 $code = sprintf("   | {$this->comment_prefix} %-59s |\n", "{$this->name} <{$this->email}>");
		 $prefix = "        ";

		 return $code;
	 }

	 /**
		* Generate a phpinfo line for this author
		*
		* @access public
		* @return string phpinfo() line
		*/
	 function phpinfo()
	 {
		 return "  php_printf(\"<p>{$this->name} &lt;{$this->email}&gt; ({$this->role})</p>\\n\");\n";
	 }

	 

	 /**
		* Generate a package.xml <maintainer> entry for this author
		*
		* @access public
		* @return string phpinfo() line
		*/
	 function package_xml()
	 {
		 $code = "    <maintainer>\n";
		 $code.= "      <user>{$this->user}</user>\n";
		 $code.= "      <name>{$this->name}</name>\n";
		 $code.= "      <email>{$this->email}</email>\n";
		 $code.= "      <role>{$this->role}</role>\n";
		 $code.= "    </maintainer>\n";
		 
		 return $code;
	 }
}

?>