<?php

/**
 * A helper class for platform management
 *
 * @access public
 */
class PECL_Gen_Tools_Platform
	extends PEAR
{
	/**
	 * The supported platforms are collected here
	 *
	 * @access private
	 * @var    array   the platform short names
	 */
	var $platforms = array();

	/**
	 * Constructor gets a list of names or "all"
	 *
	 * @access public
	 * @param  string|array names as comma separated string or array
	 */
	function __construct($names) 
  {
	  if (is_string($names)) {
		  $names = explode(",", $names);
    }

		foreach ($names as $name) {
			switch (strtolower(trim($name))) {
			case "all":
				$this->platforms["win"] = true;
				$this->platforms["unix"] = true;
				break;

			case "win":
			case "win23":
			case "windows":
			case "microsoft":
				$this->platforms["win"] = true;
				break;

			case "unix":
			case "posix":
			case "gnu":
				$this->platforms["unix"] = true;
				break;

			default:
				$this = $this->raiseError("'$name' is not a supported platform");
				break(2);
			}
		}
	}

	/**
	 * Test for a platform shortname
	 *
	 * @access public
	 * @param  string shortname
	 * @return bool   true if supported else false
	 */
	function test($name) 
	{
		switch (strtolower(trim($name))) {
		case "all":
			return 2 == $this->count(); 
			
		case "win":
		case "win23":
		case "windows":
		case "microsoft":
			return isset($this->platforms["win"]);
			
		case "unix":
		case "posix":
		case "gnu":
			return ($this->platforms["unix"]);
			
		default:
			return false;
		}
	}

	/** 
	 * Count the number of supported platforms
	 *
	 * @access public
	 * @return int    platform count
	 */
	function count()
	{
		return count($this->platforms());
	}
}

?>